<?php
add_action('plugins_loaded', 'init_giftcard_gateway_class');

function init_giftcard_gateway_class() {

    class GiftCard_Gateway extends WC_Payment_Gateway {

        private $_remove_failed_orders;

        public function __construct() {
            $this->id = 'ws_voucher';
            $this->method_title = __('Voucher Payments', 'woocommerce-zc-giftcard');
            $this->method_description = __('Allows payment using gift card coupons.', 'woocommerce-zc-giftcard');
            $this->has_fields = true;
            $this->enabled = true;
            $this->_remove_failed_orders = ($this->get_option('remove_failed_order') == 'yes');
            $this->supports = apply_filters('wooznd_wallet_gateway_supports', array(
                'products'
            ));

//		$this->supports           	= array(
//			'products',
//			'tokenization',
//			'subscriptions',
//			'subscription_cancellation',
//			'subscription_suspension',
//			'subscription_reactivation',
//			'subscription_amount_changes',
//			'subscription_date_changes',
//			'subscription_payment_method_change',
//			'subscription_payment_method_change_customer'
//		);

            $this->init_form_fields();
            $this->init_settings();


            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');

            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        }

        /**
         * Initialise Gateway Settings Form Fields
         */
        public function init_form_fields() {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('Enable/Disable', 'woocommerce-zc-giftcard'),
                    'type' => 'checkbox',
                    'label' => __('Enable Voucher Payment', 'woocommerce-zc-giftcard'),
                    'default' => 'no'
                ),
                'title' => array(
                    'title' => __('Title', 'woocommerce-zc-giftcard'),
                    'type' => 'text',
                    'description' => __('This controls the title which the user sees during checkout.', 'woocommerce-zc-giftcard'),
                    'default' => __('Voucher Payment', 'woocommerce-zc-giftcard'),
                    'desc_tip' => true,
                ),
                'low_message' => array(
                    'title' => __('Insufficient Funds Text', 'woocommerce-zc-giftcard'),
                    'type' => 'textarea',
                    'description' => __('This controls the insufficient funds message which the user sees when someone tries to puchase item without having enough money in their gift card.', 'woocommerce-zc-giftcard'),
                    'default' => __('Insufficient funds.', 'woocommerce-zc-giftcard'),
                    'desc_tip' => true,
                ),
                'description' => array(
                    'title' => __('Description', 'woocommerce-zc-giftcard'),
                    'description' => __('This controls the description which the user sees during checkout.', 'woocommerce-zc-giftcard'),
                    'type' => 'textarea',
                    'default' => 'You can make payments using gift card voucher',
                    'desc_tip' => true,
                ),
            );
        }

        function payment_fields() {
            $coupon = '';
            if (isset($_POST['giftcard_coupon'])) {
                $coupon = sanitize_text_field($_POST['giftcard_coupon']);
            }
            ?>
            <div class="woo_wallet_panel">
                <div class="woo_wallet_message">
                    <?php
                    echo $this->description;
                    ?>
                </div>
                <br />
                <div class="ws_pay_input">
                    <input type="password" name="giftcard_coupon" value="<?php echo esc_attr($coupon); ?>" />
                    <!--                    <br /><br />
                                        <input type="submit" value="Check" />-->
                </div>
            </div>
            <?php
        }

        public function process_payment($order_id) {
            global $woocommerce;
            $order = new WC_Order($order_id);
            $total = $order->calculate_totals();
            $coupon = sanitize_text_field($_POST['giftcard_coupon']);
            $card = WooZndGiftCardDB::GetGiftCardByCode($coupon);

            $card_balance = $card['coupon_amount'];


            if ($card_balance >= $total) {
                // Mark as on-hold (we're awaiting the cheque)
               // $order->update_status('on-hold', __('Completing wallet payment transaction', 'woocommerce-zc-giftcard'));

                if (WooZndGiftCardDB::DebitGiftCardAmount($coupon, $total)) {

                    $order->payment_complete($coupon);
                }
                $woocommerce->cart->empty_cart();

                // Return thankyou redirect
                return array(
                    'result' => 'success',
                    'redirect' => $this->get_return_url($order)
                );
            } else {
                if ($this->_remove_failed_orders == true) {
                    wp_delete_post($order_id, true);
                }
                wc_add_notice($this->get_option('low_message'), 'error');
                return;
            }
        }

//        public function process_refund($order_id, $amount = null, $reason = '') {
//            return (apply_filters('wooznd_wallet_process_refund', $amount, $reason, $order_id) > 0);
//        }
    }

}
