<?php
add_action('plugins_loaded', 'init_woozndwallet_gateway_class');

function init_woozndwallet_gateway_class() {

    class WooZndWallet_Gateway extends WC_Payment_Gateway {

        private $_wallet;
        private $_user; //need to fix this, some admin users may not be able to create order with this payment method for users.
        private $_remove_failed_orders;

        public function __construct() {
            $this->id = 'wooznd_wallet';
            $this->method_title = __('Wallet Payments', 'smartpack');
            $this->method_description = __('Allows payment using funds in the wallet database.', 'smartpack');
            $this->has_fields = true;
            $this->enabled = true;
            $this->_remove_failed_orders = ($this->get_option('remove_failed_order') == 'yes');
            $this->supports = apply_filters('wooznd_wallet_gateway_supports', array(
                'products'
            ));

//		$this->supports           	= array(
//			'products',
//			'tokenization',
//			'subscriptions',
//			'subscription_cancellation',
//			'subscription_suspension',
//			'subscription_reactivation',
//			'subscription_amount_changes',
//			'subscription_date_changes',
//			'subscription_payment_method_change',
//			'subscription_payment_method_change_customer'
//		);
            if (!is_user_logged_in()) {
                $this->enabled = false;
                return;
            }


            $this->_wallet = WooZndWalletAccountDB::GetAccount(get_current_user_id());
            $this->_user = wp_get_current_user();



            if (is_checkout()) {
                if (!isset($this->_wallet['id'])) {
                    $this->enabled = false;
                    return;
                }
            }

            $this->init_form_fields();
            $this->init_settings();


            $this->title = preg_replace('/{{funds}}/', wc_price($this->_wallet['current_balance']), $this->get_option('title'));


            if (isset($this->_wallet['locked']) && $this->_wallet['locked'] == true) {
                $this->description = $this->get_option('lock_message');
            } else {
                $this->description = $this->get_option('description');
            }


            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        }

        /**
         * Initialise Gateway Settings Form Fields
         */
        public function init_form_fields() {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('Enable/Disable', 'smartpack'),
                    'type' => 'checkbox',
                    'label' => __('Enable Wallet Payment', 'smartpack'),
                    'default' => 'yes'
                ),
                'title' => array(
                    'title' => __('Title', 'smartpack'),
                    'type' => 'text',
                    'description' => __('This controls the title which the user sees during checkout.', 'smartpack'),
                    'default' => __('My Wallet', 'smartpack'),
                    'desc_tip' => true,
                ),
                'description' => array(
                    'title' => __('Description', 'smartpack'),
                    'description' => __('This controls the description which the user sees during checkout.', 'smartpack'),
                    'type' => 'textarea',
                    'default' => 'Make payment using {{funds}} available funds in your wallet',
                    'desc_tip' => true,
                ),
                'lock_message' => array(
                    'title' => __('Lock Text', 'smartpack'),
                    'type' => 'textarea',
                    'description' => __('This controls the lock message which the user sees when their wallet is locked.', 'smartpack'),
                    'default' => __('Your wallet account is currently locked please contact our customer care.', 'smartpack'),
                    'desc_tip' => true,
                ),
                'low_message' => array(
                    'title' => __('Insufficient Funds Text', 'smartpack'),
                    'type' => 'textarea',
                    'description' => __('This controls the insufficient funds message which the user sees when someone tries to puchase item without having enough money in their wallet.', 'smartpack'),
                    'default' => __('Insufficient funds.', 'smartpack'),
                    'desc_tip' => true,
                ),
                'remark' => array(
                    'title' => __('Transaction Remarks', 'smartpack'),
                    'type' => 'text',
                    'description' => __('This controls transaction remarks made by the gateway.', 'smartpack'),
                    'default' => __('Order Payment', 'smartpack'),
                    'desc_tip' => true,
                ),
                'remove_failed_order' => array(
                    'title' => __('Remove Pending Orders', 'smartpack'),
                    'type' => 'checkbox',
                    'label' => __('Auto Remove Pending Payment Orders', 'smartpack'),
                    'default' => 'no'
                ),
            );
        }

        function payment_fields() {
            ?>
            <div class="woo_wallet_panel<?php echo ($this->_wallet['locked'] == true) ? ' locked' : '' ?>">
                <?php
                if ($this->_wallet['locked'] == true) {
                    ?>
                    <i class="dashicons dashicons-lock"></i>
                    <?php
                }
                ?>
                <div class="woo_wallet_message">
                    <?php
                    echo preg_replace('/{{funds}}/', wc_price($this->_wallet['current_balance']), $this->description);
                    ?>
                </div>          
            </div>
            <?php
        }

        public function process_payment($order_id) {
            global $woocommerce;
            $order = new WC_Order($order_id);
            $order_user = $order->get_user();
            $user_login = $order_user->user_login;
            $user_id = $order_user->ID;

            $total = $order->calculate_totals();
            $wallet_balance = $this->_wallet['current_balance'];

            if ($this->_wallet['locked'] == true) {
                if ($this->_remove_failed_orders == true) {
                    wp_delete_post($order_id, true);
                }
                wc_add_notice($this->get_option('lock_message'), 'error');
                return;
            }

            $znd_found = false;
            foreach (WC()->cart->get_fees() as $fee) {
                if ($fee->tax_class == 'znd_wallet_partial_payment') {
                    $znd_found = true;
                }
            }

            if ($znd_found == true) {
                wc_add_notice($this->get_option('low_message'), 'error');
                return;
            }


            if ($wallet_balance >= $total) {
                // Mark as on-hold (we're awaiting the cheque)
               // $order->update_status('on-hold', __('Completing wallet payment transaction', 'smartpack'));



                $trans_id = WooZndWalletTransactionDB::DebitWallet($user_id, $total, ZND_WALLET_TRANSANCTION_PAYMENT, $user_login, $this->get_option('remark'));
                if ($trans_id > 0) {
                    $compp = WooZndWalletTransactionDB::TransactionComplete($trans_id, WooZndUtil::GetOption('system_login', 'system_login'), $this->get_option('remark'));
                    WooZndWalletTransactionDB::SetTransactionOrderId($trans_id, $order_id);

                    if ($compp > 0) {
                        $order->payment_complete(WooZndWalletTransactionDB::GetTransactionReceipt($trans_id));
                    }
                } else if ($total == 0) {
                    $order->payment_complete();
                }

                // Reduce stock levels
                // $order->reduce_order_stock();
                // Remove cart
                $woocommerce->cart->empty_cart();

                // Return thankyou redirect
                return array(
                    'result' => 'success',
                    'redirect' => $this->get_return_url($order)
                );
            } else {
                if ($this->_remove_failed_orders == true) {
                    wp_delete_post($order_id, true);
                }
                wc_add_notice($this->get_option('low_message'), 'error');
                return;
            }
        }

        public function process_refund($order_id, $amount = null, $reason = '') {
            return (apply_filters('wooznd_wallet_process_refund', $amount, $reason, $order_id) > 0);
        }

    }

}
